/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.beans.view;

import java.io.IOException;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.analysis.model.analysisMetaModel.MIView;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.beans.application.GlobalPropertiesBean;
import kieker.webgui.beans.application.ProjectsBean;
import kieker.webgui.common.ClassAndMethodContainer;
import kieker.webgui.common.IProjectManager;
import kieker.webgui.common.exception.DisplayNotFoundException;
import kieker.webgui.common.exception.ProjectNotExistingException;

/**
 * The {@link CurrentCockpitBean} contains the necessary data behind an instance of the cockpit. It provides methods to read the state of the currently selected
 * project.<br>
 * The class is a Spring managed bean with view scope to make sure that one user (even in one session) can open multiple projects at a time without causing any
 * problems.
 * 
 * @author Nils Christian Ehmke
 */
public final class CurrentCockpitBean {

	private static final Log LOG = LogFactory.getLog(CurrentCockpitBean.class);
	private IProjectManager projectManagerFacade;
	private String projectName;
	private MIProject project;
	private MIView activeView;
	private ClassAndMethodContainer classAndMethodContainer;
	private ProjectsBean projectsBean;

	/**
	 * Creates a new instance of this class. <b>Do not call this constructor manually. It will only be accessed by Spring.</b>
	 */
	public CurrentCockpitBean() {
		// No code necessary
	}

	/**
	 * This method delivers the project stored in this bean.
	 * 
	 * @return The project for this user.
	 */
	public MIProject getProject() {
		synchronized (this) {
			return this.project;
		}
	}

	public void setProjectManagerFacade(final IProjectManager projectManagerFacade) {
		this.projectManagerFacade = projectManagerFacade;
	}

	/**
	 * The getter for the {@link #projectsBean}-property.
	 * 
	 * @return {@link #projectsBean}
	 */
	public ProjectsBean getProjectsBean() {
		synchronized (this) {
			return this.projectsBean;
		}
	}

	/**
	 * The setter for the {@link #projectsBean}-property.
	 * 
	 * @param projectsBean
	 *            The new value for {@link #projectsBean}.
	 */
	public void setProjectsBean(final ProjectsBean projectsBean) {
		synchronized (this) {
			this.projectsBean = projectsBean;
		}
	}

	/**
	 * This method sets the project stored within this bean.
	 * 
	 * @param newName
	 *            The name of the project.
	 */
	public void setProjectName(final String newName) {
		synchronized (this) {
			// Remember the given parameters
			this.projectName = newName;
		}
	}

	/**
	 * This method initializes the bean by using the current project name to load the project. <b>Do not call this method manually. It will only be accessed by
	 * Spring.</b>
	 */
	public void initalize() {
		// TODO Replace the log messages with the localizes messages
		synchronized (this) {
			try {
				// Make sure that the initialization will only be done for the init request.
				if (!FacesContext.getCurrentInstance().isPostback()) {
					// Remember the given parameters
					this.project = this.projectsBean.openProject(this.projectName);

					if (this.project != null) {
						final ClassLoader classLoader = this.projectManagerFacade.getClassLoader(this.projectName); // NOPMD (ClassLoader)
						this.classAndMethodContainer = new ClassAndMethodContainer(classLoader);
					}
				}
			} catch (final ProjectNotExistingException ex) {
				CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
				GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			} catch (final IOException ex) {
				CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
				GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			} catch (final NullPointerException ex) {
				// This exception occurs, when the projectsBean has not been initialized
				CurrentCockpitBean.LOG.error("An error occured while loading the project.", ex);
				GlobalPropertiesBean.showMessage(FacesMessage.SEVERITY_ERROR, "An error occured while loading the project.");
			}
		}
	}

	/**
	 * This method delivers the project name stored in this bean.
	 * 
	 * @return The project name for this user.
	 */
	public String getProjectName() {
		synchronized (this) {
			return this.projectName;
		}
	}

	/**
	 * Delivers the current content for the given display as plain text.
	 * 
	 * @param displayName
	 *            The name of the display.
	 * @return The current content of the display, if it exists. If the display does not exist or the active view is not set 'N/A' will be returned. If an access to
	 *         the display fails, "Error" will be returned.
	 */
	public String updatePlainTextDisplay(final String displayName) {
		synchronized (this) {
			if ((this.activeView != null) && (this.projectName != null)) {
				try {
					final Object displayObj = this.projectManagerFacade.getDisplay(this.projectName, this.activeView.getName(), displayName);
					final String result = (String) ClassAndMethodContainer.invokeMethod(this.classAndMethodContainer.getPlainTextgetTextMethod(), displayObj,
							"Error");
					return result;
				} catch (final DisplayNotFoundException ex) {
					CurrentCockpitBean.LOG.warn("Display not found.", ex);
				}
			}
			return "N/A";
		}
	}

	/**
	 * Delivers the current content for the given display as html text.
	 * 
	 * @return Currently this method returns only N/A.
	 */
	public String updateHtmlTextDisplay() {
		return "N/A";
	}

	/**
	 * Delivers the current content for the given display as Image link.
	 * 
	 * @return Currently this method returns only N/A.
	 */
	public String updateImageDisplay() {
		return "N/A";
	}

	/**
	 * Delivers the currently active view.
	 * 
	 * @return The active view.
	 */
	public MIView getActiveView() {
		synchronized (this) {
			return this.activeView;
		}
	}

	/**
	 * Sets the active view to a new value.
	 * 
	 * @param activeView
	 *            The new active view.
	 */
	public void setActiveView(final MIView activeView) {
		synchronized (this) {
			this.activeView = activeView;
		}
	}
}
