/***************************************************************************
 * Copyright 2012 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.common.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.sql.DataSource;

import org.springframework.security.access.prepost.PreAuthorize;

import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;
import kieker.webgui.common.IUserManager;
import kieker.webgui.common.Role;
import kieker.webgui.common.User;

public class UserManagerImpl implements IUserManager {

	private static final Log LOG = LogFactory.getLog(UserManagerImpl.class);
	private DataSource dataSource;
	private Connection connection;

	/**
	 * Default constructor. <b>Do not use this constructor. This bean is Spring managed.</b>
	 */
	public UserManagerImpl() {
		// No code necessary
	}

	/**
	 * This method initializes the object.. <b>Do not call this method manually. It will only be accessed by Spring.</b>
	 */
	public void initialize() {
		try {
			this.connection = this.dataSource.getConnection();
		} catch (final SQLException ex) {
			UserManagerImpl.LOG.error("Could not establish database connection.", ex);
		}
	}

	public void destroy() {
		try {
			this.connection.close();
		} catch (final SQLException ex) {
			UserManagerImpl.LOG.error("Could not close database connection.", ex);
		}
	}

	public void setDataSource(final DataSource dataSource) {
		this.dataSource = dataSource;
	}

	@Override
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	public void addUser(final String username, final String password, final List<Role> roles) {
		PreparedStatement userCmd = null;
		PreparedStatement roleCmd = null;
		try {
			userCmd = this.connection.prepareStatement("INSERT INTO KIEKERUser (name, password, enabled) VALUES (?, ?, True)");
			roleCmd = this.connection.prepareStatement("INSERT INTO Userroles (name, role) VALUES (?, ?)");

			userCmd.setString(1, username);
			userCmd.setString(2, password);
			userCmd.execute();

			roleCmd.setString(1, username);
			for (final Role role : roles) {
				roleCmd.setInt(2, role.getID());
				roleCmd.execute();
			}
		} catch (final SQLException ex) {
			UserManagerImpl.LOG.error("Could not add user to the database.", ex);
		} finally {
			if (userCmd != null) {
				try {
					userCmd.close();
				} catch (final SQLException ex) {
					UserManagerImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}
			if (roleCmd != null) {
				try {
					roleCmd.close();
				} catch (final SQLException ex) {
					UserManagerImpl.LOG.error("Could not close prepared statement.", ex);
				}
			}
		}
	}

	@Override
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	public void removeUser(final String username) {
		// TODO Auto-generated method stub

	}

	@Override
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	public void editUser(final String username, final String password, final Role... roles) {
		// TODO Auto-generated method stub

	}

	@Override
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	public List<User> getUsers() {
		// FIXME Users without roles
		final List<User> result = new ArrayList<User>();
		ResultSet queryResult = null;

		try {
			final Map<String, User> tempMap = new TreeMap<String, User>();

			final PreparedStatement getQuery = this.connection
					.prepareStatement("select u.name, ur.role, u.enabled from KIEKERUser u, Userroles ur where u.name=ur.name");

			// Run through all results
			queryResult = getQuery.executeQuery();
			while (queryResult.next()) {
				// Get both the username and the role from the current entry
				final String username = queryResult.getString(1);
				final int roleID = queryResult.getInt(2);
				final Role role = Role.fromID(roleID);
				final boolean enabled = queryResult.getBoolean(3);

				// If the user doesn't exist in our map yet, add him.
				// In each case we add the role to the user
				if (tempMap.containsKey(username)) {
					tempMap.get(username).getRoles().add(role);
				} else {
					final List<Role> roles = new ArrayList<Role>();
					roles.add(role);
					tempMap.put(username, new User(username, roles, enabled));
				}
			}

			// Now convert the map to the list
			result.addAll(tempMap.values());
		} catch (final SQLException ex) {
			UserManagerImpl.LOG.error("Could not receive user list.", ex);
		} finally {
			try {
				if (queryResult != null) {
					queryResult.close();
				}
			} catch (final SQLException ex) {
				UserManagerImpl.LOG.error("Could not close query result.", ex);
			}
		}

		return result;
	}
}
